@@ -3,7 +3,7 @@ import contextlib
 import torch
 
 from pytorch_pfn_extras._cupy import cupy
-from pytorch_pfn_extras._cupy import ensure_cupy
+from pytorch_pfn_extras._cupy import is_available, ensure_cupy
 
 
 _allocator = None
@@ -23,7 +23,7 @@ def stream(stream):
         return
 
     with torch.cuda.stream(stream):
-        if cupy is not None:
+        if is_available():
             cupy_stream = cupy.cuda.ExternalStream(stream.cuda_stream)
             with cupy_stream:
                 yield
@@ -61,7 +61,7 @@ def _torch_alloc(size, device_id):
             ' Use `pytorch_pfn_extras.cuda.stream` instead of'
             ' `torch.cuda.stream`.')
     return torch.cuda.caching_allocator_alloc(
-            size, device_id, torch_stream_ptr)
+        size, device_id, torch_stream_ptr)
 
 
 def _torch_free(mem_ptr, device_id):
