@@ -1,4 +1,5 @@
 import inspect
+from typing import Tuple
 import warnings
 
 import torch
@@ -31,8 +32,8 @@ class LazyInitializationMixin:
 
     # Subclasses must override these fields and list names of all buffers /
     # parameters that will be initialized lazily.
-    lazy_buffer_names = ()
-    lazy_parameter_names = ()
+    lazy_buffer_names: Tuple[str, ...] = ()
+    lazy_parameter_names: Tuple[str, ...] = ()
 
     def __init__(self, *args, **kwargs):
         self._lazy_ready = False
@@ -80,32 +81,45 @@ class LazyInitializationMixin:
             missing_keys, unexpected_keys, error_msgs):
         """load_state_dict pre-hook function for lazy buffers and parameters.
 
-        The purpose of this hook is to adjust the current state and/or
-        ``state_dict`` being loaded so that a module instance serialized in
-        both un/initialized state can be deserialized onto both un/initialized
-        module instance.
+        The purpose of this hook is to check the current state and/or
+        ``state_dict`` being loaded and ensure that both are states
+        are properly initialized.
 
         See comment in ``torch.nn.Module._register_load_state_dict_pre_hook``
         for the details of the hook specification.
         """
         for name in self.lazy_buffer_names:
-            # Avoid shape mismatch error when loading an initialized buffer
-            # onto an uninitialized module instance.
-            self.register_buffer(name, state_dict[prefix + name])
+            key = prefix + name
+            module_initialized = getattr(self, name).shape != (0,)
+            state_initialized = state_dict[key].shape != (0,)
+            if module_initialized and not state_initialized:
+                raise RuntimeError(
+                    'Can\'t load non-initialized buffers in already '
+                    'initialized modules')
+            elif not module_initialized and state_initialized:
+                # Here we need to avoid a tensor size mismatch
+                # this is a regular tensor without a materialize
+                # method, so we can just resize for the load logic to copy
+                # the contents later to the correct device the module
+                # was moved to
+                getattr(self, name).resize_(state_dict[key].size())
 
         for name in self.lazy_parameter_names:
-            # The parameter may not exist in the loaded ``state_dict`` if the
+            # The parameter does not exist in the loaded ``state_dict`` if the
             # original module was serialized before initializing lazy
             # parameters (see comments of ``state_dict``).
             key = prefix + name
-            if key in state_dict:
-                # The model was serialized after initialization.
-                self.register_parameter(
-                     name, torch.nn.Parameter(state_dict[key]))
-            else:
-                # The model was serialized before initialization.
+            module_initialized = not isinstance(
+                getattr(self, name), UninitializedParameter)
+            state_initialized = key in state_dict
+            if module_initialized and not state_initialized:
+                raise RuntimeError(
+                    'Can\'t load uninitialized parameters in already '
+                    'initialized modules')
+            elif not module_initialized and state_initialized:
+                getattr(self, name).materialize(state_dict[key].shape)
+            elif key not in state_dict and not module_initialized:
                 param = UninitializedParameter()
-                self.register_parameter(name, param)
                 state_dict[key] = param
 
 
@@ -132,3 +146,25 @@ class UninitializedParameter(torch.nn.Parameter):
     Use of uninitialized lazy parameter in Optimizer has been detected.
     Maybe you forgot to run forward before passing `module.parameters()` to the optimizer?''')  # NOQA
         return True
+
+    def materialize(self, shape, device=None, dtype=None):
+        r"""Create a Parameter with the same properties of the uninitialized
+        one. Given a shape, it materializes a parameter in the same device
+        and with the same `dtype` as the current one or the specified ones in
+        the arguments.
+
+        Args:
+            shape : (tuple): the shape for the materialized tensor.
+            device (:class:`torch.device`): the desired device of the
+                parameters
+                and buffers in this module. Optional.
+            dtype (:class:`torch.dtype`): the desired floating point type of
+                the floating point parameters and buffers in this module.
+                Optional.
+        """
+        if device is None:
+            device = self.data.device
+        if dtype is None:
+            dtype = self.data.dtype
+        self.data = torch.empty(shape, device=device, dtype=dtype)
+        self.__class__ = torch.nn.Parameter
