@@ -1,3 +1,4 @@
+import collections
 import json
 
 from pytorch_pfn_extras import reporting
@@ -33,6 +34,12 @@ class LogWriterSaveFunc:
             if self._append:
                 target = [target[-1]]
             import yaml
+
+            # This is to dump ordered dicts as regular dicts
+            def dict_representer(dumper, data):
+                return dumper.represent_dict(data.items())
+            yaml.add_representer(collections.OrderedDict, dict_representer)
+            # yaml.add_constructor(_mapping_tag, dict_constructor)
             log = yaml.dump(target)
         else:
             raise ValueError('Unknown format: {}'.format(self._format))
@@ -84,6 +91,10 @@ keys=None, trigger=(1, 'epoch'), postprocess=None, filename='log', writer=None)
             does not output the log to any file.
             For historical reasons ``log_name`` is also accepted as an alias
             of this argument.
+        append (bool, optionsl): If the file is JSON Lines or YAML, contents
+            will be appended instead of rewritting the file every call.
+        format (str, optional): accepted values are `'json'`, `'json-lines'`
+            and `'yaml'`.
         writer (writer object, optional): must be callable.
             object to dump the log to. If specified, it needs to have a correct
             `savefun` defined. The writer can override the save location in
