@@ -5,6 +5,7 @@ import queue
 import shutil
 import sys
 import threading
+from typing import Optional
 
 import torch
 
@@ -68,8 +69,11 @@ class _PosixFileSystem(object):
                        buffering, encoding, errors,
                        newline, closefd, opener)
 
-    def list(self, path_or_prefix: str = None, recursive=False):
+    def list(self, path_or_prefix: Optional[str] = None, recursive=False):
         if recursive:
+            if path_or_prefix is None:
+                raise ValueError(
+                    "'path_or_prefix' must not be None in recursive mode.")
             path_or_prefix = path_or_prefix.rstrip("/")
             # plus 1 to include the trailing slash
             prefix_end_index = len(path_or_prefix) + 1
@@ -115,7 +119,7 @@ class _PosixFileSystem(object):
             return os.replace(src, dst)
         except OSError:
             print('Destination {} is a directory '
-                  'but source is not'.format(src),
+                  'but source is not'.format(dst),
                   file=sys.stderr)
             raise
 
@@ -156,7 +160,8 @@ class Writer:
 
         self._initialized = False
 
-    def __call__(self, filename, out_dir, target):
+    def __call__(self, filename, out_dir, target, *,
+                 savefun=None, append=False):
         """Invokes the actual snapshot function.
 
         This method is invoked by a
@@ -171,6 +176,12 @@ class Writer:
                 :py:attr:`ExtensionsManager.out
                  <pytorch_pfn_extras.training.ExtensionsManager.out>`.
             target (dict): Serialized object which will be saved.
+            savefun (callable): A callable that accepts a two positional
+                arguments (an object to be serialized, file path) like
+                `torch.save`.
+            append (bool): Mode used to open the file. True to use the append
+                mode, False to use the write mode (truncates the file if it
+                already exists).
         """
         raise NotImplementedError
 
@@ -182,15 +193,11 @@ class Writer:
         self.finalize()
 
     def finalize(self):
-        """Finalizes the writer.
-
-        this method is invoked at the end of the training in
-        :class:`~pytorch_pfn_extras.training.ExtensionsManager`,
-
-        """
+        """Finalizes the writer."""
         pass
 
-    def save(self, filename, out_dir, target, savefun, append, **kwds):
+    def save(self, filename, out_dir, target,
+             savefun, append, **savefun_kwargs):
         if self.out_dir is not None:
             out_dir = self.out_dir
         if not self._initialized:
@@ -201,18 +208,23 @@ class Writer:
         if append:
             with self.fs.open(dest, 'ab') as f:
                 # HDFS does not support overwrite
-                savefun(target, f)
+                savefun(target, f, **savefun_kwargs)
         else:
             # Some filesystems are not compatible with temp folders, etc
             # so we rely on raw temp files
             prefix = 'tmp_{}'.format(filename)
             tmppath = os.path.join(out_dir, prefix)
             make_backup = self.fs.exists(dest)
+            with self.fs.open(tmppath, 'wb') as f:
+                savefun(target, f, **savefun_kwargs)
             if make_backup:
                 bak = '{}.bak'.format(dest)
+                # Check if another backup file exists
+                # due to some unexpected termination of an earlier
+                # process
+                if self.fs.exists(bak):
+                    self.fs.remove(bak)
                 self.fs.rename(dest, bak)
-            with self.fs.open(tmppath, 'wb') as f:
-                savefun(target, f)
             self.fs.rename(tmppath, dest)
             if make_backup:
                 self.fs.remove(bak)
@@ -292,51 +304,50 @@ class StandardWriter(Writer):
         - :meth:`pytorch_pfn_extras.training.extensions.snapshot`
     """
 
-    _started = False
-    _finalized = False
-    _worker = None
-
     def __init__(self, savefun=torch.save, fs=None, out_dir=None, **kwds):
         super().__init__(fs=fs, out_dir=out_dir)
         self._savefun = savefun
         self._kwds = kwds
+        self._worker = None
         self._started = False
         self._finalized = False
 
-    def __call__(self, filename, out_dir, target, *, savefun=None):
+    def __call__(self, filename, out_dir, target, *,
+                 savefun=None, append=False):
         if savefun is None:
             savefun = self._savefun
         if self._started:
-            self._worker.join()
-            self._started = False
+            self.finalize()
         self._filename = filename
-        self._worker = self.create_worker(filename, out_dir, target,
-                                          savefun=savefun, **self._kwds)
+        self._worker = self.create_worker(
+            filename, out_dir, target,
+            savefun=savefun, append=append, **self._kwds)
         self._worker.start()
         self._started = True
+        self._finalized = False
 
-    def create_worker(self, filename, out_dir, target, savefun, **kwds):
+    def create_worker(
+            self, filename, out_dir, target, *,
+            savefun=None, append=False, **savefun_kwargs):
         """Creates a worker for the snapshot.
 
         This method creates a thread or a process to take a snapshot. The
         created worker must have :meth:`start` and :meth:`join` methods.
-
-        Args:
-            filename (str): Name of the file into which the serialized target
-                is saved. It is already formated string.
-            out_dir (str): Output directory. Passed by `manager.out`.
-            target (dict): Serialized object which will be saved.
-            kwds: Keyword arguments for the ``savefun``.
-
+        If the worker has an ``exitcode`` attribute (e.g.,
+        ``multiprocessing.Process``), the value will be tested.
         """
         raise NotImplementedError
 
     def finalize(self):
-        if self._started:
-            if not self._finalized:
+        try:
+            if self._started and not self._finalized:
                 self._worker.join()
+                exitcode = getattr(self._worker, 'exitcode', 0)
+                if exitcode != 0:
+                    raise RuntimeError(f'exit code is non-zero: {exitcode}')
+        finally:
             self._started = False
-        self._finalized = True
+            self._finalized = True
 
 
 class ThreadWriter(StandardWriter):
@@ -352,11 +363,23 @@ class ThreadWriter(StandardWriter):
     def __init__(self, savefun=torch.save, fs=None, out_dir=None, **kwds):
         super().__init__(savefun=savefun, fs=fs, out_dir=out_dir, **kwds)
 
-    def create_worker(self, filename, out_dir, target, **kwds):
+    def _save_with_exitcode(self, *args, **kwargs):
+        try:
+            self.save(*args, **kwargs)
+        except Exception as e:
+            thread = threading.current_thread()
+            thread.exitcode = -1
+            print(
+                f'Error: ThreadWriter failed in thread "{thread.name}": '
+                f'{type(e).__name__}: {str(e)}', file=sys.stderr)
+
+    def create_worker(
+            self, filename, out_dir, target, *,
+            savefun=None, append=False, **savefun_kwargs):
         return threading.Thread(
-            target=self.save,
-            args=(filename, out_dir, target, self._savefun),
-            kwargs=self._kwds)
+            target=self._save_with_exitcode,
+            args=(filename, out_dir, target, savefun, append),
+            kwargs=savefun_kwargs)
 
 
 class ProcessWriter(StandardWriter):
@@ -377,11 +400,13 @@ class ProcessWriter(StandardWriter):
     def __init__(self, savefun=torch.save, fs=None, out_dir=None, **kwds):
         super().__init__(savefun=savefun, fs=fs, out_dir=out_dir, **kwds)
 
-    def create_worker(self, filename, out_dir, target, **kwds):
+    def create_worker(
+            self, filename, out_dir, target, *,
+            savefun=None, append=False, **savefun_kwargs):
         return multiprocessing.Process(
             target=self.save,
-            args=(filename, out_dir, target, self._savefun),
-            kwargs=self._kwds)
+            args=(filename, out_dir, target, savefun, append),
+            kwargs=savefun_kwargs)
 
 
 class QueueWriter(Writer):
@@ -426,8 +451,10 @@ class QueueWriter(Writer):
         self._started = True
         self._finalized = False
 
-    def __call__(self, filename, out_dir, target, *, savefun=None):
-        self._queue.put([self._task, filename, out_dir, target, savefun])
+    def __call__(
+            self, filename, out_dir, target, *, savefun=None, append=False):
+        self._queue.put(
+            [self._task, filename, out_dir, target, savefun, append])
 
     def create_task(self, savefun):
         return SimpleWriter(savefun=savefun)
@@ -445,7 +472,8 @@ class QueueWriter(Writer):
                 q.task_done()
                 return
             else:
-                task[0](task[1], task[2], task[3], savefun=task[4])
+                task[0](
+                    task[1], task[2], task[3], savefun=task[4], append=task[5])
                 q.task_done()
 
     def finalize(self):
@@ -515,6 +543,12 @@ class TensorBoardWriter(object):
     object that is used to send the collected statistics to TensorBoard.
     A list of stats can be specified to report only the desired ones.
 
+    Args:
+        savefun: Ignored.
+        fs: Ignored.
+        out_dir: Passed as ``log_dir`` argument to SummaryWriter.
+        stats (list): List of statistic keys.
+        kwds: Passed as an additional arguments to SummaryWriter.
     """
     def __init__(
             self, savefun=None, fs=None, out_dir=None, stats=None, **kwds):
@@ -523,9 +557,23 @@ class TensorBoardWriter(object):
         self._writer = torch.utils.tensorboard.SummaryWriter(
             log_dir=out_dir, **kwds)
 
-    def __call__(self, filename, out_dir, target, *, savefun=None):
+    def __del__(self):
+        self.finalize()
+
+    def __call__(
+            self, filename, out_dir, target, *, savefun=None, append=False):
+        """Sends the statistics to the TensorBoard.
+
+        Args:
+            filename: Ignored.
+            out_dir: Ignored.
+            target (dict or list): The statistics of the iteration. If given as
+                a list, only the last element (assumed to be a dict containing
+                the latest iteration statistics) is reported.
+            savefun: Ignored.
+            append: Ignored.
+        """
         stats_cpu = target
-        # we only take the last value
         if isinstance(target, list):
             stats_cpu = target[-1]
 
@@ -537,3 +585,6 @@ class TensorBoardWriter(object):
         for key in keys:
             value = stats_cpu[key]
             self._writer.add_scalar(key, value, stats_cpu['iteration'])
+
+    def finalize(self):
+        self._writer.close()
